@@ -4,6 +4,8 @@ import re
 import os
 import sys
 import copy
+import time
+import signal
 import hashlib
 import subprocess
 import ConfigParser
@@ -11,7 +13,7 @@ from optparse import OptionParser, OptionGroup
 
 from PIL import Image as PImage
 
-__version__ = '0.2.4'
+__version__ = '0.2.5'
 
 TRANSPARENT = (255, 255, 255, 0)
 
@@ -24,6 +26,7 @@ PSEUDO_CLASSES = set(['link', 'visited', 'active', 'hover', 'focus',
                       'before', 'after'])
 
 DEFAULT_SETTINGS = {'padding': '0',
+                    'margin': '0',
                     'algorithm': 'square',
                     'ordering': 'maxside',
                     'namespace': 'sprite',
@@ -31,6 +34,7 @@ DEFAULT_SETTINGS = {'padding': '0',
                     'url': '',
                     'less': False,
                     'optipng': False,
+                    'html': False,
                     'ignore_filename_paddings': False,
                     'png8': False,
                     'separator': '-',
@@ -41,6 +45,20 @@ DEFAULT_SETTINGS = {'padding': '0',
                                       'background-position:%(x)s %(y)s;'
                                       'width:%(width)s;height:%(height)s;}\n')}
 
+TEST_HTML_TEMPLATE = """
+<html><head><title>Glue Sprite Test Html</title>
+<link rel="stylesheet" type="text/css" href="%(css_url)s"></head><body>
+<style type="text/css">tr div:hover{ border:1px solid #ccc;}
+tr div{ border:1px solid white;}</style><h1>CSS Classes</h1><table>
+<tr><th>CSS Class</th><th>Result</th></tr>%(sprites)s</table>
+<p><em>Generated using <a href="http://gluecss.com"/>Glue v%(version)s</a>
+</em></p></body></html>
+"""
+
+TEST_HTML_SPRITE_TEMPLATE = """
+<tr><td>.%(class_name)s </td><td><div class="%(class_name)s"></div></td></tr>
+"""
+
 
 class MultipleImagesWithSameNameError(Exception):
     """Raised if two images pretend to generate the same CSS class name."""
@@ -299,7 +317,8 @@ class Image(object):
 
             if source_image.mode == 'L':
                 alpha = source_image.split()[0]
-                mask = PImage.eval(alpha, lambda a: 0 if a == source_image.info.get('transparency', 0) else 255)
+                transparency = source_image.info.get('transparency')
+                mask = PImage.eval(alpha, lambda a: 0 if a == transparency else 255)
                 self.image.paste(source_image, (0, 0), mask=mask)
             else:
                 self.image.paste(source_image, (0, 0))
@@ -313,8 +332,9 @@ class Image(object):
             self._crop_image()
 
         self.width, self.height = self.image.size
-        self.absolute_width = self.width + self.padding[1] + self.padding[3]
-        self.absolute_height = self.height + self.padding[0] + self.padding[2]
+        margin = int(self.sprite.config.margin)
+        self.absolute_width = self.width + self.padding[1] + self.padding[3] + (margin * 2)
+        self.absolute_height = self.height + self.padding[0] + self.padding[2] + (margin * 2)
 
     def _crop_image(self):
         """Crop the image searching for the smallest possible bounding box
@@ -506,8 +526,8 @@ class Sprite(object):
         """
         extensions = '|'.join(VALID_IMAGE_EXTENSIONS)
         extension_re = re.compile('.+\.(%s)$' % extensions, re.IGNORECASE)
-
-        images = [Image(n, sprite=self) for n in os.listdir(self.path) if \
+        files = sorted(os.listdir(self.path))
+        images = [Image(n, sprite=self) for n in files if \
                                     not n.startswith('.') and \
                                     extension_re.match(n)]
 
@@ -546,9 +566,10 @@ class Sprite(object):
         canvas = PImage.new('RGBA', (width, height), (0, 0, 0, 0))
 
         # Paste the images inside the canvas
+        margin = int(self.config.margin)
         for image in self.images:
-            canvas.paste(image.image, (image.x + image.padding[3],
-                                       image.y + image.padding[0]))
+            canvas.paste(image.image, (image.x + image.padding[3] + margin,
+                                       image.y + image.padding[0] + margin))
 
         if self.config.cachebuster or self.config.cachebuster_filename:
             self.cachebuster_hash = hashlib.sha1(canvas.tostring()
@@ -611,12 +632,13 @@ class Sprite(object):
                                    'sprite_url': self.image_url})
 
         # compile one template for each file
+        margin = int(self.config.margin)
         for image in self.images:
 
-            x = '%spx' % (image.x * -1 if image.x else 0)
-            y = '%spx' % (image.y * -1 if image.y else 0)
-            height = '%spx' % image.absolute_height
-            width = '%spx' % image.absolute_width
+            x = '%spx' % ((image.x * -1 if image.x else 0) - margin)
+            y = '%spx' % ((image.y * -1 if image.y else 0) - margin)
+            height = '%spx' % (image.absolute_height - (margin * 2))
+            width = '%spx' % (image.absolute_width - (margin * 2))
 
             template = self.config.each_template.decode('unicode-escape')
             css_file.write(template % {'class_name': '.%s' % image.class_name,
@@ -627,6 +649,34 @@ class Sprite(object):
                                        'x': x})
         css_file.close()
 
+    def save_html(self):
+        self.manager.log("Creating '%s' html file..." % self.name)
+
+        output_path = self.manager.output_path('css')
+        filename = '%s.html' % self.filename
+        html_filename = os.path.join(output_path, filename)
+
+        # CSS output format
+        format = 'less' if self.config.less else 'css'
+
+        # Fix css urls on Windows
+        html_filename = '/'.join(html_filename.split('\\'))
+
+        html_file = open(html_filename, 'w')
+
+        # get all the class names and join them
+        class_names = [i.class_name for i in self.images \
+                                                if ':' not in i.class_name]
+
+        sprite_template = TEST_HTML_SPRITE_TEMPLATE.decode('unicode-escape')
+        sprites_html = [sprite_template % {'class_name':c} for c in class_names]
+
+        file_template = TEST_HTML_TEMPLATE.decode('unicode-escape')
+        html_file.write(file_template % {'sprites': ''.join(sprites_html),
+                                         'css_url': '%s.%s' % (self.filename, format),
+                                         'version': __version__})
+        html_file.close()
+
     @property
     def namespace(self):
         """Return the namespace for this sprite."""
@@ -695,8 +745,8 @@ class ConfigManager(object):
 
         :param config: Dictionary with the new config.
         """
-        return self.__class__(config, *self.sources, priority=self.priority,
-                              defaults=self.defaults)
+        return self.__class__(config, priority=self.priority,
+                              defaults=self.defaults, *self.sources)
 
     def __getattr__(self, name):
         """Return the first available configuration value for this key. This
@@ -745,6 +795,8 @@ class BaseManager(object):
         for sprite in self.sprites:
             sprite.save_image()
             sprite.save_css()
+            if sprite.manager.config.html:
+                sprite.save_html()
 
     def output_path(self, format):
         """Return the path where all the generated files will be saved.
@@ -828,6 +880,41 @@ class SimpleSpriteManager(BaseManager):
         self.save()
 
 
+class WatchManager(object):
+
+    def __init__(self, path, action):
+        self.action = action
+        self.path = path
+        self.last_hash = None
+
+    def run(self):
+        signal.signal(signal.SIGINT, self.signal_handler)
+
+        while True:
+            try:
+                current_hash = self.generate_hash()
+                if self.last_hash != current_hash:
+                    self.action()
+                self.last_hash = current_hash
+            except Exception:
+                pass
+            finally:
+                time.sleep(0.2)
+
+    def signal_handler(self, signal, frame):
+        print 'You pressed Ctrl+C!'
+        sys.exit(0)
+
+    def generate_hash(self):
+        hash_list = []
+        for root, dirs, files in os.walk(self.path):
+            for f in sorted([f for f in files if not f.startswith('.')]):
+                hash_list.append(os.path.join(root, f))
+                hash_list.append(str(os.path.getmtime(os.path.join(root, f))))
+        hash_list = ''.join(hash_list)
+        return hashlib.sha1(hash_list).hexdigest()
+
+
 def get_file_config(path, section='sprite'):
     """Return, as a dictionary, all the available configuration inside the
     sprite configuration file on this path.
@@ -918,6 +1005,10 @@ def main():
                       help="suppress all normal output")
     parser.add_option("-p", "--padding", dest="padding", default=None,
                       help="force this padding in all images")
+    parser.add_option("-w", "--watch", dest="watch", default=False,
+                      action='store_true',
+                      help=("Watch the source folder for changes and rebuild "
+                            "when new files appear, disappear or change."))
     parser.add_option("-v", "--version", action="store_true", dest="version",
                       help="show program's version number and exit")
 
@@ -926,6 +1017,9 @@ def main():
                     help="output directory for css files", metavar='DIR')
     group.add_option("--img", dest="img_dir", default='', metavar='DIR',
                     help="output directory for img files")
+    group.add_option("--html", dest="html", default=None, action="store_true",
+                    help=("generate test html file using the sprite "
+                          "image and CSS."))
     parser.add_option_group(group)
 
     group = OptionGroup(parser, "Advanced Options")
@@ -935,6 +1029,8 @@ def main():
     group.add_option("--ordering", dest="ordering", default=None,
                     help=("ordering criteria: maxside, width, height or "
                           "area (default: maxside)"), metavar='NAME')
+    group.add_option("--margin", dest="margin", default=None,
+                      help="force this margin in all images", type=int)
     group.add_option("--namespace", dest="namespace",  default=None,
                       help="namespace for all css classes (default: sprite)")
     group.add_option("--png8", action="store_true", dest="png8",
@@ -1030,6 +1126,10 @@ def main():
     config = ConfigManager(config, priority=options, defaults=DEFAULT_SETTINGS)
     manager = manager_cls(path=source, output=output, config=config)
 
+    if manager.config.watch:
+        WatchManager(path=source, action=manager.process).run()
+        sys.exit(0)
+
     try:
         manager.process()
     except MultipleImagesWithSameNameError, e:
